#ifndef H_EXPATPP
#define H_EXPATPP

#include "xmlparse.h"

/*
expatpp follows a simple pattern for converting the semi-OOP callback design of 
expat into a true class which allows you to override virtual methods to supply
callbacks.

USING expatpp
see testexpatpp.cpp for a detailed example

1) decide which callbacks you wish to use, eg: just startElement

2) declare a subclass of expatpp, eg:
class myExpat : public expatpp {
	virtual void startElement(const XML_Char* name, const XML_Char** atts);
};

3) create an instance of your object and pass in a buffer to parse
myExpat parser;
parser.XML_Parse(buf, len, done)


HOW IT WORKS
The User Data which expat maintains is simply a pointer to an instance of your object.

Inline static functions are specified as the callbacks to expat.
These static functions take the user data parameter returned from expat and cast it
to a pointer to an expatpp object.

Using that typed pointer they then call the appropriate virtual method.

If you have overriden a given virtual method then your version will be called, otherwise
the (empty) method in the base expatpp class is called.

NOTE:
For efficiency, you could provide your own constructor and set some of the callbacks
to 0, so expat doesn't call the static functions. (untested idea).

*/


class expatpp {
public:
	expatpp();
	~expatpp();

	operator XML_Parser() const;

// overrideable callbacks
	virtual void startElement(const XML_Char* name, const XML_Char** atts);
	virtual void endElement(const XML_Char* name);
	virtual void charData(const XML_Char *s, int len);
	virtual void processingInstruction(const XML_Char* target, const XML_Char* data);
	virtual void defaultHandler(const XML_Char *s, int len);
	virtual void unparsedEntityDecl(const XML_Char *entityName, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId, const XML_Char* notationName);
	virtual void notationDecl(const XML_Char* notationName, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId);
	
// XML interfaces
	int XMLPARSEAPI XML_Parse(const char *s, int len, int isFinal);
	XML_Error XMLPARSEAPI XML_GetErrorCode();
	int XMLPARSEAPI XML_GetCurrentLineNumber();
	
protected:
	XML_Parser mParser;

// interface functions for callbacks
public:
	static void startElementCallback(void *userData, const XML_Char* name, const XML_Char** atts);
	static void endElementCallback(void *userData, const XML_Char* name);
	static void charDataCallback(void *userData, const XML_Char* s, int len);
	static void processingInstructionCallback(void *userData, const XML_Char* target, const XML_Char* data);
	static void defaultHandlerCallback(void* userData, const XML_Char* s, int len);
	static void unParsedEntityDeclCallback(void* userData, const XML_Char* entityName, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId, const XML_Char* notationName);
	static void notationDeclCallback(void *userData, const XML_Char* notationName, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId);
	
// utilities
	static int skipWhiteSpace(const char*);	
};

// inlines

inline void 
expatpp::startElementCallback(void *userData, const XML_Char* name, const XML_Char** atts)
{
	((expatpp*)userData)->startElement(name, atts);
}


inline void 
expatpp::endElementCallback(void *userData, const XML_Char* name)
{
	((expatpp*)userData)->endElement(name);
}


inline void 
expatpp::charDataCallback(void *userData, const XML_Char* s, int len)
{
	((expatpp*)userData)->charData(s, len);
}

inline void
expatpp:: processingInstructionCallback(void *userData, const XML_Char* target, const XML_Char* data)
{
	((expatpp*)userData)->processingInstruction(target, data);
}

inline void
expatpp::defaultHandlerCallback(void* userData, const XML_Char* s, int len)
{
	((expatpp*)userData)->defaultHandler(s, len);
}

inline void
expatpp::unParsedEntityDeclCallback(void* userData, const XML_Char* entityName, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId, const XML_Char* notationName)
{
	((expatpp*)userData)->unparsedEntityDecl(entityName, base, systemId, publicId, notationName);
}

inline void
expatpp::notationDeclCallback(void *userData, const XML_Char* notationName, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId)
{
	((expatpp*)userData)->notationDecl(notationName, base, systemId, publicId);
}

//inline int
//expatpp::externalEntityRefCallback(XML_Parser parser, const XML_Char* openEntityNames, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId)
//{
//	((expatpp*)parser)->externalEntityRef(openEntityNames, base, systemId, publicId);
//}

inline 	
expatpp::operator XML_Parser() const
{
	return mParser;
}


inline int XMLPARSEAPI 
expatpp::XML_Parse(const char *s, int len, int isFinal)
{
	return ::XML_Parse(mParser, s, len, isFinal);
}


inline XML_Error 
expatpp::XMLPARSEAPI XML_GetErrorCode()
{
	return ::XML_GetErrorCode(mParser);
}


inline int XMLPARSEAPI 
expatpp::XML_GetCurrentLineNumber()
{
	return ::XML_GetCurrentLineNumber(mParser);
}

#endif

